import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

/**
 * The hole in the grid.
 * 
 * @author David J. Barnes
 * @version 2008.08.26
 */
public class Hole extends Actor
{
    /**
     * Create a hole.
     */
    public Hole()
    {
    }

    /**
     * Return a list of tiles adjacent to the hole.
     * @return The adjacent tiles.
     */
    public List<Tile> getAdjacentTiles()
    {
        List<Tile> adjacent = new ArrayList<Tile>();
        int x = getX(), y = getY();
        addAdjacent(x - 1, y, adjacent);
        addAdjacent(x, y - 1, adjacent);
        addAdjacent(x + 1, y, adjacent);
        addAdjacent(x, y + 1, adjacent);
        return adjacent;
    }
    
    /**
     * Add the tile at position (x,y) - if it exists - to the list.
     * @param x The x-coordinate of the tile.
     * @param y The y-coordinate of the tile.
     * @param adjacent The list to add the tile to.
     */
    private void addAdjacent(int x, int y, List<Tile> adjacent)
    {
        if(validLocation(x, y)) {
            List<Actor> tile = getWorld().getObjectsAt(x, y, Tile.class);
            adjacent.add((Tile) tile.get(0));
        }
    }
    
    /**
     * Is (x,y) a valid location in the grid?
     * @param x The x-coordinate of the location.
     * @param y The y-coordinate of the location.
     * @return true If the location is valid, false otherwise.
     */
    private boolean validLocation(int x, int y)
    {
        return x >= 0 && x < FifteenWorld.SQUARE_SIDE &&
               y >= 0 && y < FifteenWorld.SQUARE_SIDE;
    }
}
