import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;

/**
 * A tile in the square.
 * When a tile is added to the world it is assumed to
 * be added at its goal position.
 * 
 * @author David J. Barnes
 * @version 2008.08.26
 */
public class Tile extends Actor
{
    // The hole in the square.
    private Hole hole;
    // The home position of this tile.
    private int goalX, goalY;
    
    /**
     * Create a tile.
     * @param hole The hole in the grid.
     */
    public Tile(Hole hole)
    {
        this.hole = hole;
    }
        
    /**
     * If the mouse is clicked on the tile, move it into the
     * adjacent hole.
     */
    public void act() 
    {
        if(Greenfoot.mouseClicked(this)) {
            if(nextToHole()) {
                switchWithHole();
            }
        }
    }
    
    /**
     * Is this tile next to the hole?
     * @return true If the tile is next to the hole, false otherwise.
     */
    public boolean nextToHole()
    {
        int holeX = hole.getX(), holeY = hole.getY();
        int x = getX(), y = getY();
        return holeX == x && Math.abs(y - holeY) == 1 ||
               holeY == y && Math.abs(x - holeX) == 1;
    }
    
    /**
     * Switch this tile with the hole.
     */
    public void switchWithHole()
    {
        int x = getX(), y = getY();
        setLocation(hole.getX(), hole.getY());
        hole.setLocation(x, y);
    }
    
    /**
     * Is this tile in its home position?
     * @return true if the tile is in position, false otherwise.
     */
    public boolean inPosition()
    {
        return getX() == goalX && getY() == goalY;
    }

    /**
     * Note that this tile has been added to the world.
     * This is used to record the tile's home position.
     */
    protected void addedToWorld(World world)
    {
        goalX = getX();
        goalY = getY();
    }
    
}
